@@ -1,5 +1,5 @@
-# Publishers
+""" Publishers package. """
 from .confluence_sender import ConfluenceSender
 from .email_sender import EmailSender
-from .slack_sender import SlackSender
 from .gdrive_uploader import GoogleDriveUploader
+from .slack_sender import SlackSender
