@@ -1,19 +1,22 @@
+""" Email sender module """
 import base64
+import os
+import smtplib
 from datetime import datetime
+from email.mime.image import MIMEImage
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
-from email.mime.image import MIMEImage
-import os
 
-from apiclient import discovery
 import httplib2
-import smtplib
+from apiclient import discovery
 from oauth2client import client, tools, file
 
-from ..utils.reporter_utils import format_error
+from ..utils.reporter_utils import format_error, check_captions_and_files
 
 
 class EmailSender:
+    """Email sender class"""
+
     def __init__(self, email=None, password=None, server_smtp=None, server_port=None, gmail_token=None, debug=True):
         """
         General init
@@ -30,13 +33,13 @@ class EmailSender:
         self.__debug = debug
         self.__method = None
         if email is not None and password is not None and server_smtp is not None and server_port is not None:
-            self.__method = 'regular'
+            self.__method = "regular"
         elif gmail_token and email:
-            gmail_token = f'{os.getcwd()}/{gmail_token}' if not os.path.exists(gmail_token) else gmail_token
+            gmail_token = f"{os.getcwd()}/{gmail_token}" if not os.path.exists(gmail_token) else gmail_token
             if os.path.exists(gmail_token):
-                self.__method = 'gmail'
-                self.__gmail_scopes = 'https://www.googleapis.com/auth/gmail.send'
-                self.__gmail_app_name = 'Gmail API Python Send Email'
+                self.__method = "gmail"
+                self.__gmail_scopes = "https://www.googleapis.com/auth/gmail.send"
+                self.__gmail_app_name = "Gmail API Python Send Email"
         if not self.__method:
             raise ValueError("No email credentials are provided, aborted!")
         self.__email = email
@@ -45,8 +48,19 @@ class EmailSender:
         self.__server_port = server_port
         self.__gmail_token = gmail_token
 
-    def send_message(self, files=None, captions=None, image_width="400px", title=None, timestamp=None, recipients=None,
-                     method=None, custom_message=None, use_home_folder=True, debug=None):
+    def send_message(  # pylint: disable=too-many-branches
+        self,
+        files=None,
+        captions=None,
+        image_width="400px",
+        title=None,
+        timestamp=None,
+        recipients=None,
+        method=None,
+        custom_message=None,
+        custom_folder=os.path.join(os.path.expanduser("~"), ".credentials"),
+        debug=None,
+    ):
         """
         Send email to recipients with report (with attached images)
 
@@ -58,7 +72,7 @@ class EmailSender:
         :param recipients: list of recipient emails, list of strings, optional
         :param method: method which will be used for sending
         :param custom_message: custom message, prepared by user at his own, by default its payload with TR state report
-        :param use_home_folder: use home folder for gmail credentials storage, by default True, otherwise current folder
+        :param custom_folder: custom home folder for gmail credentials storage, by default is ~/.credentials
         :param debug: debug output is enabled, may be True or False, optional
         :return: none
         """
@@ -72,32 +86,33 @@ class EmailSender:
         elif not isinstance(recipients, list) and not custom_message:
             raise ValueError("Wrong list of recipients is provided, aborted!")
         debug = debug if debug is not None else self.__debug
-        if not isinstance(captions, list) or custom_message:
+        captions = check_captions_and_files(captions=captions, files=files, debug=debug)
+        if not captions or custom_message:
             if debug:
-                print("Caption list is empty, no legend will be displayed")
-            captions = None
-        elif len(captions) != len(files):
-            if debug:
-                print(f"Caption and file lists are not the same length {len(captions)} != {len(files)} thus "
-                      f"no legend will be displayed")
-            captions = None
+                print("Caption list override by custom message, no legend will be displayed")
         timestamp = timestamp if timestamp else datetime.now().strftime("%Y-%m-%d")
         title = title if title else f"Test development & automation coverage report for {timestamp}"
 
         # Connect and send message
         if not custom_message:
-            message = self.__prepare_payload(files=files, captions=captions, image_width=image_width, title=title,
-                                             recipients=recipients, method=method)
+            message = self.__prepare_payload(
+                files=files,
+                captions=captions,
+                image_width=image_width,
+                title=title,
+                recipients=recipients,
+                method=method,
+            )
         else:
             if debug:
                 print("Ignoring payload preparations, assuming user custom message is right")
             message = custom_message
-        if method == 'regular':
+        if method == "regular":
             connection = self.__connect_to_server()
             self.__send_to_server(connection=connection, recipients=recipients, message=message)
             self.__disconnect_from_server(connection=connection)
-        elif method == 'gmail':
-            self.__gmail_send_message(message=message, use_home_folder=use_home_folder)
+        elif method == "gmail":
+            self.__gmail_send_message(message=message, custom_folder=custom_folder)
         if debug:
             print("Email sent!")
 
@@ -108,14 +123,14 @@ class EmailSender:
         :return: connection handle ( smtplib.SMTP )
         """
         if self.__debug:
-            print(f'Connecting to custom mail server {self.__server_smtp}:{self.__server_port} using {self.__email}')
+            print(f"Connecting to custom mail server {self.__server_smtp}:{self.__server_port} using {self.__email}")
         try:
             connection = smtplib.SMTP(self.__server_smtp, self.__server_port)
             connection.ehlo()
             connection.starttls()
             connection.login(self.__email, self.__password)
-        except Exception as e:
-            raise ValueError(f"Can't login to mail!\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Can't login to mail!\nError{format_error(error)}") from error
         return connection
 
     def __send_to_server(self, connection, recipients, message):
@@ -128,11 +143,11 @@ class EmailSender:
         :return: none
         """
         if self.__debug:
-            print(f'Sending mail from {self.__email} to {recipients}')
+            print(f"Sending mail from {self.__email} to {recipients}")
         try:
             connection.sendmail(from_addr=self.__email, to_addrs=recipients, msg=message.as_string())
-        except Exception as e:
-            raise ValueError(f"Can't send mail!\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Can't send mail!\nError{format_error(error)}") from error
 
     def __disconnect_from_server(self, connection):
         """
@@ -142,11 +157,11 @@ class EmailSender:
         :return: none
         """
         if self.__debug:
-            print(f'Disconnecting from custom server')
+            print("Disconnecting from custom server")
         try:
             connection.quit()
-        except Exception as e:
-            raise ValueError(f"Can't close connection!\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Can't close connection!\nError{format_error(error)}") from error
 
     def __prepare_payload(self, files, image_width, title, recipients, captions=None, method=None):
         """
@@ -160,110 +175,114 @@ class EmailSender:
         :param method: specify which method is used to set proper MIMEMultipart type ('gmail' or not)
         :return: formatted multipart message
         """
-        message = MIMEMultipart("alternative") if method != 'gmail' else MIMEMultipart()
+        message = MIMEMultipart("alternative") if method != "gmail" else MIMEMultipart()
         message["Subject"] = title
         message["From"] = self.__email
         message["To"] = ", ".join(recipients)
-        html = '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8">' \
-               f'<title>{title}</title></head>' \
-               f'<body><div align="center"><h3>{title}' \
-               '</h3></div><table border="0px" width="100%"><tbody><td>'
+        html = (
+            '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8">'
+            f"<title>{title}</title></head>"
+            f'<body><div align="center"><h3>{title}'
+            '</h3></div><table border="0px" width="100%"><tbody><td>'
+        )
         for j, val in enumerate(files):
-            with open('{}'.format(val), "rb") as attachment:
+            with open(f"{val}", "rb") as attachment:
                 mime_image = MIMEImage(attachment.read())
 
             # Define the image's ID with counter as you will reference it.
-            mime_image.add_header('Content-ID', f'<image_id_{j}>')
-            mime_image.add_header('Content-Disposition', f'attachment; filename= {val}')
+            mime_image.add_header("Content-ID", f"<image_id_{j}>")
+            mime_image.add_header("Content-Disposition", f"attachment; filename= {val}")
             message.attach(mime_image)
             # add to body
             if captions:
                 html = f'{html}<tr><div align="center"><b>{captions[j]}</b></div></tr>'
-            html = f'{html}<tr><div align="center"><img src="cid:image_id_{j}" ' \
-                   f'width="{image_width}" height="auto">></div></tr>'
-        html = f'{html}</td></tbody></table></body></html>'
+            html = (
+                f'{html}<tr><div align="center"><img src="cid:image_id_{j}" '
+                f'width="{image_width}" height="auto">></div></tr>'
+            )
+        html = f"{html}</td></tbody></table></body></html>"
         message.attach(MIMEText(html, "html"))
         return message
 
-    def __gmail_get_credential_path(self, use_home_folder):
+    def __gmail_get_credential_path(self, custom_folder=os.path.join(os.path.expanduser("~"), ".credentials")):
         """
         Service function target Google OAuth credentials path to storage
 
-        :param use_home_folder: use home folder for gmail credentials storage, by default True, otherwise current folder
+        :param custom_folder: custom home folder for gmail credentials storage, by default is ~/.credentials
         :return: credentials file path (string)
         """
-        if use_home_folder:
-            credential_dir = os.path.join(os.path.expanduser('~'), '.credentials')
-        else:
-            credential_dir = os.path.abspath(os.path.join(__file__, os.pardir))
         if self.__debug:
-            print(f"Checking GMail credentials path at {credential_dir}")
+            print(f"Checking GMail credentials path at {custom_folder}")
         try:
             if self.__debug:
-                print(f"No credential directory found, creating new one here: {credential_dir}")
-            os.makedirs(credential_dir, exist_ok=True)
-        except OSError as e:
+                print(f"No credential directory found, creating new one here: {custom_folder}")
+            os.makedirs(custom_folder, exist_ok=True)
+        except OSError as error:
             if self.__debug:
-                print(f"Original Error{format_error(e)}")
-        credential_path = os.path.join(credential_dir, 'gmail-python-email-send.json')
+                print(f"Original Error{format_error(error)}")
+        credential_path = os.path.join(custom_folder, "gmail-python-email-send.json")
         return credential_path
 
-    def __gmail_get_credentials(self, use_home_folder):
+    def __gmail_get_credentials(self, custom_folder=os.path.join(os.path.expanduser("~"), ".credentials")):
         """
         Service function to get and convert Google OAuth credential from client_id and client_secret
 
-        :param use_home_folder: use home folder for gmail credentials storage, by default True, otherwise current folder
+        :param custom_folder: custom home folder for gmail credentials storage, by default is ~/.credentials
         :return: credentials
         """
-        credential_path = self.__gmail_get_credential_path(use_home_folder=use_home_folder)
+        credential_path = self.__gmail_get_credential_path(custom_folder=custom_folder)
         if self.__debug:
             print(f"Obtaining GMail credentials from {credential_path}")
         try:
             store = file.Storage(credential_path)
-        except Exception as e:
-            raise ValueError(f"Couldn't open storage\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Couldn't open storage\nError{format_error(error)}") from error
         try:
             credentials = store.get()
-        except Exception as e:
-            raise ValueError(f"Obtaining of credentials unexpectedly failed\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Obtaining of credentials unexpectedly failed\nError{format_error(error)}") from error
         if not credentials or credentials.invalid:
             try:
                 flow = client.flow_from_clientsecrets(self.__gmail_token, self.__gmail_scopes)
-            except Exception as e:
-                raise ValueError(f"Couldn't obtain new client secrets from Google OAuth\nError{format_error(e)}")
+            except Exception as error:
+                raise ValueError(
+                    f"Couldn't obtain new client secrets from Google OAuth\nError{format_error(error)}"
+                ) from error
             flow.user_agent = self.__gmail_app_name
             try:
                 credentials = tools.run_flow(flow, store)
-            except Exception as e:
-                raise ValueError(f"Couldn't obtain new credential from Google OAuth\nError{format_error(e)}")
+            except Exception as error:
+                raise ValueError(
+                    f"Couldn't obtain new credential from Google OAuth\nError{format_error(error)}"
+                ) from error
             if self.__debug:
-                print('Credentials stored to ' + credential_path)
+                print("Credentials stored to " + credential_path)
         return credentials
 
-    def __gmail_send_message(self, message, use_home_folder):
+    def __gmail_send_message(self, message, custom_folder=os.path.join(os.path.expanduser("~"), ".credentials")):
         """
         Send Email via GMail
 
         :param message: message in MIME type format
-        :param use_home_folder: use home folder for gmail credentials storage, by default True, otherwise current folder
+        :param custom_folder: custom home folder for gmail credentials storage, by default is ~/.credentials
         :return: none
         """
         if self.__debug:
             print("Sending message using GMail")
-        credentials = self.__gmail_get_credentials(use_home_folder=use_home_folder)
+        credentials = self.__gmail_get_credentials(custom_folder=custom_folder)
         try:
             http = credentials.authorize(httplib2.Http())
-        except Exception as e:
-            raise ValueError(f"Can't authorize via Google OAuth\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Can't authorize via Google OAuth\nError{format_error(error)}") from error
         try:
-            service = discovery.build('gmail', 'v1', http=http)
-        except Exception as e:
-            raise ValueError(f"Can't build service for Google OAuth\nError{format_error(e)}")
+            service = discovery.build("gmail", "v1", http=http)
+        except Exception as error:
+            raise ValueError(f"Can't build service for Google OAuth\nError{format_error(error)}") from error
         try:
             raw = base64.urlsafe_b64encode(message.as_bytes()).decode()
-        except Exception as e:
-            raise ValueError(f"Can't convert payload to base64\nError{format_error(e)}")
-        self.__gmail_send_message_internal(service, self.__email, {'raw': raw})
+        except Exception as error:
+            raise ValueError(f"Can't convert payload to base64\nError{format_error(error)}") from error
+        self.__gmail_send_message_internal(service, self.__email, {"raw": raw})
 
     def __gmail_send_message_internal(self, service, user_id, message):
         """
@@ -275,9 +294,9 @@ class EmailSender:
         :return: message
         """
         try:
-            message = (service.users().messages().send(userId=user_id, body=message).execute())
+            message = service.users().messages().send(userId=user_id, body=message).execute()
             if self.__debug:
-                print(f'Message Id: {message["id"]}')
+                print(f'Message sent with Id: "{message["id"]}"!')
             return message
-        except Exception as e:
-            raise ValueError(f"Can't send mail via GMail!\nError{format_error(e)}")
+        except Exception as error:
+            raise ValueError(f"Can't send mail via GMail!\nError{format_error(error)}") from error
